unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  XDOM_2_3, StdCtrls, ComCtrls;

type
  TrpIteratorFilter = Class(TdomNodeFilter)
    function acceptNode(const n: TdomNode): TdomFilterResult; override;
  end;

  TrpTreeWalkerFilter = Class(TdomNodeFilter)
    function acceptNode(const n: TdomNode): TdomFilterResult; override;
  end;

  TForm1 = class(TForm)
    btnLoadXML: TButton;
    DomImpl: TDomImplementation;
    XmlToDomParser: TXmlToDomParser;
    btnPopulate: TButton;
    ListView1: TListView;
    Button1: TButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Memo2: TMemo;
    Label1: TLabel;
    btnIterator: TButton;
    Memo1: TMemo;
    TabSheet2: TTabSheet;
    ListBox1: TListBox;
    btnParseNodeType: TButton;
    Label2: TLabel;
    TabSheet3: TTabSheet;
    Label3: TLabel;
    Memo3: TMemo;
    btnTreeWalker: TButton;
    Label4: TLabel;
    Memo4: TMemo;
    procedure btnLoadXMLClick(Sender: TObject);
    procedure btnPopulateClick(Sender: TObject);
    procedure btnIteratorClick(Sender: TObject);
    procedure btnParseNodeTypeClick(Sender: TObject);
    procedure btnTreeWalkerClick(Sender: TObject);
  private
    { Private declarations }
    doc : TDomDocument;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

(* ************************************************************************ *)

procedure TForm1.btnLoadXMLClick(Sender: TObject);
begin
    // Load the pre-prepared XML file into an XDOM...
    doc:=XmlToDomParser.fileToDom('richplum.xml');

    memo1.text := doc.codeAsString;
    memo3.text := memo1.text;

    btnPopulate.Enabled:=true;
end;

(* ************************************************************************ *)

procedure TForm1.btnPopulateClick(Sender: TObject);
var xmlStaffList        : TDomNodeList;
    xmlNodeMap          : TDomNamedNodeMap;
    xmlSurnameNode,
    xmlFirstNameNode    : TDomNode;

    liItem      : TListItem;
    sStaffNo    : String;
    i           : Integer;

    nt : TdomNodeType;
begin
    Listview1.Items.Clear;
    // 'Get' all the <STAFF> elements...
    xmlStaffList := doc.getElementsByTagName('staff');

    for i:=0 to xmlStaffList.Length - 1 do begin
        // Obtain access to the attributes for this element...
        xmlNodeMap := xmlStaffList.item(i).attributes;

        nt:=xmlNodeMap.GetNamedItem('no').NodeType;
        if nt=ntAttribute_Node THEN showmessage('a');

        sStaffNo            := xmlNodeMap.GetNamedItem('no').NodeValue;

        xmlSurnameNode      := xmlStaffList.item(i).childNodes.item(0);
        nt:=xmlSurnameNode.NodeType;
        if nt=ntElement_Node THEN showmessage('b');
        // OR xmlSurnameNode      := xmlStaffList.item(i).firstChild;

        xmlFirstNameNode    := xmlStaffList.item(i).childNodes.item(1);

        // Add the staff member to the ListView...
        liItem          := Listview1.Items.add;
        liItem.Caption  := sStaffNo; //xmlNodeMap.GetNamedItem('no').nodeValue;
        liItem.SubItems.Add(xmlSurnameNode.firstChild.nodeValue);
        liItem.SubItems.Add(xmlFirstNameNode.firstChild.nodeValue);
    end;
end;

(* ************************************************************************ *)

{procedure TForm1.Button1Click(Sender: TObject);
var xmlStaffList        : TDomNodeList;
    xmlFirstNameNode    : TDomNode;

    i           : Integer;

begin
    memo1.text := doc.documentElement.firstChild.code;
    xmlFirstNameNode := doc.documentElement.firstChild;
    MEMO1.TEXT := xmlFirstNameNode.namespaceURI;

    xmlStaffList := doc.getElementsByTagNameNS('*','*');
    //'http://poorpeach.co.uk/invoice','poorpeach');
    memo1.text := '';
    for i:=0 to xmlStaffList.Length - 1 do begin
      memo1.text:=memo1.text+xmlStaffList.item(i).code;
   end;
end;}

(* ************************************************************************ *)

function TrpIteratorFilter.acceptNode(const n: TdomNode): TdomFilterResult;
var  xmlNodeMap : TDomNamedNodeMap;
     sStaffNo   : string;
begin
    if n.nodeName = 'staff' then begin
        xmlNodeMap := n.attributes;

        sStaffNo   := xmlNodeMap.GetNamedItem('no').NodeValue;

        if (StrToInt(sStaffNo) mod 2) = 0 then
            result := filter_reject
        else
            result := filter_accept;
    end
    else
        result:=filter_reject;
end;

(* ************************************************************************ *)

procedure TForm1.btnIteratorClick(Sender: TObject);
var domItr  : TdomNodeIterator;
    filter  : TrpIteratorFilter;
    node    : TdomNode;

begin
    // Create node filter
    filter := TrpIteratorFilter.Create;

    try
        // Create node iterator
        domItr:=TdomNodeIterator.create(    doc.documentElement,
                                            [ntElement_Node],
                                            filter,
                                            false);
        memo2.clear;
        node := domItr.nextNode;
        while node <> nil do begin
            memo2.text := memo2.text + node.code + #13#10;
            node:=domItr.nextNode;
        end;
    finally
        filter.free;
    end;
end;

(* ************************************************************************ *)

procedure TForm1.btnParseNodeTypeClick(Sender: TObject);
var   xmlNode : TDomNode;

      function _TypeOf(n : TdomNode) : string;
      var sType : string;
      begin
        case n.nodeType of
          ntAttribute_Node: sType := 'ntAttribute_Node';
          ntElement_Node:   sType := 'ntElement_Node';
          ntComment_Node:   sType := 'ntComment_Node';
          ntText_Node:      sType := 'ntText_Node';
          // ...abbreviated...
        end;
        _TypeOf := sType;
      end;

      procedure _Recurse(n : TdomNode);
      begin
          while n<>nil do begin
            Listbox1.items.add(n.nodeName + '('+ n.nodeValue + ') is a ' + _TypeOf(n));
            if n.hasChildNodes then _Recurse(n.firstChild);
            n:=n.nextSibling;
          end;
      end;

begin
    Listbox1.Clear;

    xmlNode := doc.firstChild;

    while xmlNode <> nil do begin
      Listbox1.items.add(xmlNode.nodeName + '('+ xmlNode.nodeValue + ') is a ' + _TypeOf(xmlNode));
      if xmlNode.hasChildNodes then _Recurse(xmlNode.firstChild);
      xmlNode:=xmlNode.nextSibling;
    end;
end;

(* ************************************************************************ *)

function TrpTreeWalkerFilter.acceptNode(const n: TdomNode): TdomFilterResult;
var  xmlNodeMap : TDomNamedNodeMap;
     sStaffNo   : string;
begin
    if n.nodeName = 'staff' then begin
        xmlNodeMap := n.attributes;

        sStaffNo   := xmlNodeMap.GetNamedItem('no').NodeValue;

        // First two staff no. only
        if (StrToInt(sStaffNo)) < 3 then
            result := filter_accept
        else
            result := filter_reject;
    end
    else
        result:=filter_reject;
end;

(* ************************************************************************ *)

procedure TForm1.btnTreeWalkerClick(Sender: TObject);
var domTW   : TdomTreeWalker;
    filter  : TrpTreeWalkerFilter;
    node    : TdomNode;

begin
    // Create node filter
    filter := TrpTreeWalkerFilter.Create;

    try
      // Create node iterator
      domTW:=TdomTreeWalker.create(   doc.documentElement,
                                       [ntElement_Node],
                                       filter,
                                       false);
      memo4.clear;
      node := domTW.nextNode;
      while node <> nil do begin
          memo4.text := memo4.text + node.code;

          if (node.parentNode <> nil) then
            memo4.text := memo4.text + ' my parent is ' + node.parentNode.nodeName;

          memo4.text := memo4.text + #13#10;
          node:=domTW.nextNode;
      end;
    finally
      filter.Free;
    end;
end;

(* ************************************************************************ *)

end.
